/*
///////////////////////////////////////////////////////////////////////////////
//
//   Project: Altera circuit of SLIDAS 
//
//   File name: altera.v
//   Version:   4.5
//   Updated:   99/08/30 14:43:36
//
//   Author: Ph. Brodier-Yourstone, ECP division, CERN
//
//  New version 4.5 by G. Damerell, 18/8/99.  Contains the triggering modules.
//
///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////

     Module description:
     -------------------

     This is the top module of the whole Altera design.

     Apart from some LED assignments, no logic is defined in this file.
     This file defines the main structure of the design by instantiating
     sub-modules and their interconnections.

///////////////////////////////////////////////////////////////////////////////
*/


module altera   (clk66, to_be_lclk, lclk,

              // PUSH BUTTONS
                 async_inj_err2, async_inj_err3,
		 async_step2,     async_step3,

              // SWITCHES		 
		 async_ldown2,    async_ldown3,
		 async_data_err2, async_data_err3,
		 async_ctrl_err2, async_ctrl_err3,
		 async_femb2,     async_femb3,
		 async_block2,    async_block3,

              // ROTARY SWITCHES
		 async_pattern_switch, async_duty_switch,
		 async_speed_switch, async_length_switch,

		 lemo_step,

		 pattern_gen_0_, pattern_gen_1_, pattern_gen_2_, 
		 pattern_gen_3_, pattern_gen_4_, pattern_gen_5_, 
		 pattern_gen_6_, pattern_gen_7_, 
		 

		 ld, uxoff_n, ureset_n, utdo_n, udw,
		 lctrl_n, lwen_n, lderr_n, url, ldown_n,

               // LEDS
		 lclk_led, uxoff_led, reset_led, utdo_led,
		 lctrl_led, lwen_led, lderr_led, ldown_led,
		 run_led, res2_led, inj_err_led);

		 

/*
*****************************************************************************
  Input / Output declarations
*****************************************************************************
*/
		 
  input clk66, lclk;
  output to_be_lclk;
  
  input async_inj_err2, async_inj_err3, async_step2, async_step3;
  
  input async_ldown2, async_ldown3, async_data_err2, async_data_err3;
  input async_ctrl_err2, async_ctrl_err3, async_femb2, async_femb3;
  input async_block2, async_block3;
  
  input [3:0] async_pattern_switch, async_duty_switch;
  input [3:0] async_speed_switch, async_length_switch;

  input lemo_step;

  input pattern_gen_0_, pattern_gen_3_, pattern_gen_4_, pattern_gen_5_, 
	pattern_gen_6_, pattern_gen_7_;
  output pattern_gen_1_, pattern_gen_2_;

/* This probably seems ridiculous - why not just declare pattern_gen[7:0]?  
The problem is that then it has to be an inout, and we can't get the Max+plus 
thing to work with an inout.*/

  output [31:0] ld;
  input uxoff_n, ureset_n, utdo_n;
  input [1:0] udw;
  output lctrl_n, lwen_n, lderr_n;
  input [3:0] url;
  output ldown_n;

  output lclk_led, uxoff_led, reset_led, utdo_led;
  output lctrl_led, lwen_led, lderr_led, ldown_led, run_led, res2_led,
         inj_err_led;


/*
*****************************************************************************
  Bus declarations
*****************************************************************************
*/

  wire [3:0] pattern_switch, duty_switch, speed_switch, length_switch;

  wire [15:0] block_length;

  wire [9:0] selection;


  
/*
*****************************************************************************
  Sub-modules
*****************************************************************************
*/

clock_gen clock_gen1 (clk66, async_speed_switch, to_be_lclk);

//-----------------------------------------------------------------------------

ureset_synchroniser ureset_sync1(lclk, ureset_n, ureset);
  
//-----------------------------------------------------------------------------

utdo_sampler utdo_sampler1(lclk, utdo_n, ureset, utdo);
  
//-----------------------------------------------------------------------------

reset_blinker reset_blinker1 (clk66, ureset_n, reset_led);

//-----------------------------------------------------------------------------

power_on_reset p_o_r (lclk, power_on_reset_n, udw);

//-----------------------------------------------------------------------------

switches switches1(lclk,
                 inj_err,  async_inj_err2,  async_inj_err3,
		 step,     async_step2,     async_step3,
		 ldown,    async_ldown2,    async_ldown3,
		 data_err, async_data_err2, async_data_err3,
		 ctrl_err, async_ctrl_err2, async_ctrl_err3,
		 femb,     async_femb2,     async_femb3,
		 block,    async_block2,    async_block3,
		 async_pattern_switch, async_duty_switch,
		 async_speed_switch, async_length_switch,
 		 pattern_switch, duty_switch, speed_switch, length_switch,
		 sysreset);
		 
//-----------------------------------------------------------------------------

xoff_handler xoff_handler1 (lclk, sysenable, SM_sysenable, uxoff_n, femb);

//-----------------------------------------------------------------------------

duty_controller duty(lclk, sysreset, ureset, duty_switch, step,
                 lemo_step, trigger_out, clock_enable, running);

//-----------------------------------------------------------------------------

ldown_gen ldown_gen1 (lclk, power_on_reset_n, ureset, ldown,
                 pattern_switch, ldown_n);

//-----------------------------------------------------------------------------

state_machine state_machine1 (lclk, 

              // SIGNALS FROM SWITCHES AND POWER_ON_RESET
                 sysreset, power_on_reset_n, pattern_switch,

              // SIGNAL FROM XOFF_HANDLER
		 SM_sysenable,

              // SIGNAL FROM LDOWN_GEN
		 ldown_n,

              // SIGNAL FROM URESET_SYNCHRONISER
		 ureset,

              // SIGNAL FROM UTDO_SAMPLER
		 utdo,

              // SIGNALS FROM DUTY_CONTROLLER
		 clock_enable, running,
		     
              // SIGNALS TO DATA_GEN
		 walking_and_inverting_reset, random_reset,
		 block_counter_reset, trigger_type_reset, data_counter_reset,
		 error_gen_reset, pipe_reset,
		 
                 walking_and_inverting_enable, random_enable,
		 block_counter_enable, trigger_type_enable,
		 data_counter_enable, error_gen_enable,
		 
		 error_gen_start, selection, int_lwen,

	      // SIGNALS TO LENGTH_CONTROLLER
		 length_reset, new_block,

	      // SIGNALS FROM LENGTH_CONTROLLER
		 length_is_0, length_minus_3,

	      // SIGNALS TO WORD_COUNTER
		 word_count_reset, word_count_enable,

	      // SIGNALS FROM WORD_COUNTER
		 end_of_block,

	      // OUTPUTS
		 run_led, 

	      // TO TRIGGER_COUNT_CONTROLLER
		block_sent); 

//-----------------------------------------------------------------------------

length_controller length_controller1
                 (lclk, sysenable, length_reset, new_block, length_is_0, length_minus_3,
		 pattern_switch, length_switch, udw, block_length);

//-----------------------------------------------------------------------------

word_counter word_counter1 (lclk, sysenable, clock_enable, word_count_reset,
                 word_count_enable, block_length, end_of_block);

//-----------------------------------------------------------------------------

data_gen data_gen1(lclk, sysenable, pattern_switch, udw,
		 data_err, ctrl_err, block, inj_err, inj_err_led, femb,
		 (pattern_gen_0_==1)?1'b0:1'b1,  // jumper off? then 0 else 1
		                                 // If a jumper is connected
		                                 // on bit 0 of pattern gene-
		                                 // rator, control values will
		                                 // be 11... and ff... instead
		                                 // of b1... and e1...
		                                 // Feature for Erik's use.
		 
              // SIGNALS FROM STATE MACHINE
                 walking_and_inverting_reset, random_reset,
		 block_counter_reset, trigger_type_reset, data_counter_reset,
		 error_gen_reset, pipe_reset,
		 
                 walking_and_inverting_enable, random_enable,
		 block_counter_enable, trigger_type_enable,
		 data_counter_enable, error_gen_enable,

		 error_gen_start,
		 
		 selection, int_lwen,
		 
              // SIGNAL FROM LENGTH CONTROLLER
		 block_length,


	      // OUTPUTS
		 lderr_n, lwen_n, lctrl_n, ld);

//-----------------------------------------------------------------------------

internal_trigger trigger1(clk66, duty_switch, sysreset, ureset, 
			 internal_trigger);

//-----------------------------------------------------------------------------

trigger_count_controller control(lclk, sysreset, ureset, internal_trigger, 
				step, lemo_step, block_sent, 
				duty_switch, speed_switch, 
				trigger_out, busy, full, overflow);
//-----------------------------------------------------------------------------




/*
*****************************************************************************
  LED assignments
*****************************************************************************
*/

  assign #1 lclk_led = lclk;
  assign #1 ldown_led = ~ldown_n;
  assign #1 utdo_led = ~utdo_n;
  assign #1 uxoff_led = ~uxoff_n;
  assign #1 res2_led = overflow;
  assign #1 lctrl_led = ~lctrl_n;
  assign #1 lderr_led = ~lderr_n;
  assign #1 lwen_led = ~lwen_n;


// pattern_gen assignments

  assign #1 pattern_gen_1_ = busy;
  assign #1 pattern_gen_2_ = full;

endmodule



/*
	      
`ifdef exemplar

module ureset_synchroniser (lclk, ureset_n, ureset);
  input lclk, ureset_n;
  output ureset;
endmodule


module utdo_sampler (lclk, utdo_n, ureset, utdo);
  input lclk, utdo_n, ureset;
  output utdo;
endmodule

  
module clock_gen(clk66, async_speed_switch, to_be_lclk);
  input clk66;          // 66 MHz clock
  output to_be_lclk;    // Clock with the desired frequency
  input [3:0] async_speed_switch;
endmodule


module reset_blinker(clk66, ureset_n, reset_led);
  input clk66, ureset_n;
  output reset_led;
endmodule


module power_on_reset (lclk, power_on_reset_n, udw);
  input lclk;
  output power_on_reset_n;
  input [1:0] udw;
endmodule
*/


/*

module switches (lclk,
                 inj_err,  async_inj_err2,  async_inj_err3,
		 step,     async_step2,     async_step3,
		 ldown,    async_ldown2,    async_ldown3,
		 data_err, async_data_err2, async_data_err3,
		 ctrl_err, async_ctrl_err2, async_ctrl_err3,
		 femb,     async_femb2,     async_femb3,
		 block,    async_block2,    async_block3,
		 async_pattern_switch, async_duty_switch,
		 async_speed_switch, async_length_switch,
 		 pattern_switch, duty_switch, speed_switch, length_switch,
		 sysreset);

  input lclk;
  input async_inj_err2, async_inj_err3, async_step2, async_step3;
  input async_ldown2, async_ldown3, async_data_err2, async_data_err3;
  input async_ctrl_err2, async_ctrl_err3, async_femb2, async_femb3;
  input async_block2, async_block3;
  input [3:0] async_pattern_switch, async_duty_switch;
  input [3:0] async_speed_switch, async_length_switch;
  output inj_err, step, ldown, data_err, ctrl_err, femb, block;
  output [3:0] pattern_switch, duty_switch, speed_switch, length_switch;
  output sysreset;
endmodule


module xoff_handler (lclk, sysenable, sysenable2, uxoff_n, femb);
  input lclk, uxoff_n, femb;
  output sysenable, sysenable2;
endmodule


module duty_controller (lclk, sysreset, ureset, duty_switch, switch_step,
                        lemo_step, trigger_out, clock_enable, running);
  input lclk, sysreset, ureset, switch_step, lemo_step, trigger_out;
  input [3:0] duty_switch;
  output clock_enable, running;
endmodule
*/

/*
module ldown_gen (lclk, power_on_reset_n, ureset, ldown, pattern_switch,
                 ldown_n);

  input lclk, power_on_reset_n, ureset, ldown;
  input[3:0] pattern_switch;
  output ldown_n;
endmodule


module state_machine(lclk, 
                 sysreset, power_on_reset_n, pattern_switch,
		 sysenable, ldown_n,
		 ureset, utdo,
		 clock_enable, running,
		 walking_and_inverting_reset, random_reset,
		 block_counter_reset, trigger_type_reset, data_counter_reset,
		 error_gen_reset, pipe_reset,
                 walking_and_inverting_enable, random_enable,
		 block_counter_enable, trigger_type_enable,
		 data_counter_enable, error_gen_enable,
		 error_gen_start, selection, int_lwen,
		 length_reset, new_block,
		 length_is_0, length_minus_3,
		 word_count_reset, word_count_enable,
		 end_of_block, run_led, block_sent); 
  input lclk, sysreset, power_on_reset_n, sysenable, ldown_n, ureset, utdo,
          clock_enable, running;
  input [3:0] pattern_switch;
  output  walking_and_inverting_reset, random_reset,
	  block_counter_reset, trigger_type_reset, data_counter_reset,
	  error_gen_reset, pipe_reset,
	  walking_and_inverting_enable, random_enable,
	  block_counter_enable, trigger_type_enable,
	  data_counter_enable, error_gen_enable, error_gen_start;

  output [9:0] selection;
  output int_lwen, length_reset, new_block;
  input length_is_0, length_minus_3;
  output word_count_reset, word_count_enable;
  input end_of_block;
  output run_led, block_sent;
endmodule
*/

/*	 
module length_controller(lclk, sysenable, length_reset, new_block, length_is_0,
                         length_minus_3, pattern_switch, length_switch,
			 udw, block_length);

  input lclk, sysenable, length_reset, new_block;
  input [3:0] pattern_switch, length_switch;
  input [1:0] udw;
  output length_is_0, length_minus_3;
  output [15:0] block_length;
endmodule


module word_counter (lclk, sysenable, clock_enable, word_count_reset,
                     word_count_enable, block_length, end_of_block);

  input lclk, sysenable, clock_enable, word_count_enable, word_count_reset;
  input [15:0] block_length;
  output end_of_block;
endmodule
*/

/*
module data_gen (lclk, sysenable, pattern_switch, udw,
		 data_err, ctrl_err, block_basis, inj_err, inj_err_led, femb,
		 pattern0_jumper_on,
  
                 walking_and_inverting_reset, random_reset,
		 block_counter_reset, trigger_type_reset, data_counter_reset,
		 error_gen_reset, pipe_reset,
		 
                 walking_and_inverting_enable, random_enable,
		 block_counter_enable, trigger_type_enable,
		 data_counter_enable, error_gen_enable,

		 error_gen_start,

		 selection, int_lwen,
		 
		 block_length, 

		 lderr_n, lwen_n, lctrl_n, ld);

  input          lclk, sysenable;
  input          walking_and_inverting_reset, random_reset,
		 block_counter_reset, trigger_type_reset, data_counter_reset,
		 error_gen_reset, pipe_reset,
                 walking_and_inverting_enable, random_enable,
		 block_counter_enable, trigger_type_enable,
		 data_counter_enable, error_gen_enable;
  input[15:0]	 block_length;
  input[3:0]     pattern_switch;
  input[9:0]     selection;
  input          int_lwen;
  input[1:0]     udw;
  input          error_gen_start;
  input          data_err, ctrl_err, block_basis, inj_err, femb,
                 pattern0_jumper_on;
  output         lderr_n, lwen_n, lctrl_n, inj_err_led;
  output [31:0]  ld;
endmodule
*/

/*
module internal_trigger (clk66, duty_switch, sysreset, ureset, 
			internal_trigger);


  input clk66, sysreset, ureset;
  input [3:0] duty_switch;
  output internal_trigger;
endmodule

module trigger_count_controller(lclk, sysreset, ureset, internal_trigger, 
				switch_step, lemo_step, block_sent, 
				duty_switch, speed_switch, 
				trigger_out, busy, overflow);


  input lclk, sysreset, ureset, internal_trigger, switch_step, lemo_step, 
        block_sent;
  input [3:0] speed_switch, duty_switch;
  output trigger_out, busy, overflow;
endmodule


 
*/  

`include "ureset_synchroniser.v"
`include "utdo_sampler.v"
`include "clock_gen.v"
`include "reset_blinker.v"
`include "power_on_reset.v"
`include "switches.v"
`include "xoff_handler.v"
`include "duty_controller.v"
`include "ldown_gen.v"
`include "state_machine.v"
`include "length_controller.v"
`include "word_counter.v"
`include "data_gen.v"

`include "ff_w_enable.v"     // used by some modules to optimise the mapping
`include "ff_w_enable_32.v"  // of the clock enable signal

`include "internal_trigger.v"
`include "trigger_count_controller.v"
