/*
///////////////////////////////////////////////////////////////////////////////
//
//   Project: Altera circuit of SLIDAS 
//
//   File name: block_counter.v
//   Version:   4.3
//   Updated:   99/08/26 13:56:17
//
//   Author: Ph. Brodier-Yourstone, ECP division, CERN
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////

     Module description:
     -------------------

     16-bit synchronous counter with synchronous enable and synchronous reset.
    
     The reset is actually implemented as a 'load' of an external value,
     which is set to 0. This trick gives better results in terms of internal
     delays when using galileo.

     GD - 8/7/99 - changed it so that it increments by 1 instead of 93, in 
     accordance with the new data format.

///////////////////////////////////////////////////////////////////////////////
*/


module block_counter (lclk, sysenable, reset, count_up, d, q);
  input lclk, sysenable, reset, count_up;
  input [15:0] d;
  output [31:0] q;
  reg [15:0] counter;


// this module has an interface to a 32-bit world...

  assign q = {16'b0, counter};
  
  always @(posedge lclk)
    if (sysenable)
      if (reset) counter <= #1 d;
      else if (count_up)
	counter <= #1 counter + 1;
endmodule
