/*
///////////////////////////////////////////////////////////////////////////////
//
//   Project: Altera circuit of SLIDAS 
//
//   File name: clock_gen.v
//   Version:   4.2
//   Updated:   99/08/26 13:56:18
//
//   Author: Ph. Brodier-Yourstone, ECP division, CERN
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////

     Module description:
     -------------------

     This module generates the clock to be used by the rest of the design.

     clk66, a 40 MHz clock, drives a counter that is always counting.
     The five lowest bits and the four highest bits of the counter are fed into
     a multiplexer, as is clk66. With the help of the external switch that
     selects the speed, one of the counter's bits or clk66 will be selected
     to become to_be_lclk.

     to_be_lclk is connected directly to an output buffer. Externally, on
     the PCB, to_be_lclk is connected to one of the clock pins of the
     Altera. 
     
///////////////////////////////////////////////////////////////////////////////
*/


module clock_gen(clk66, async_speed_switch, to_be_lclk);
  input clk66;          // 66 MHz clock
  output to_be_lclk;    // Clock with the desired frequency
  input [3:0] async_speed_switch;


  wire [3:0] speed_switch;
  
  reg to_be_lclk;
  reg [25:0] int_clks;   // internally generated clock
  parameter
    _40_MHz   = 0,
    _20_MHz   = 1,
    _10_MHz   = 2,
    _5_MHz    = 3,
    _2_5_MHz  = 4,
    _1_25_MHz = 5,
    _40_Hz    = 6,
    _2_Hz     = 7,
    _1_Hz     = 8,
    _half_Hz  = 9;


  sync_rotary sync_rotary1(clk66, speed_switch, async_speed_switch);
  
//  always @(posedge clk66) speed_switch <= #1 ~async_speed_switch;  

  always @(posedge clk66) int_clks <= #1 int_clks + 1;

    
  always @(clk66 or int_clks or speed_switch)
    case (speed_switch)
      _40_MHz:   to_be_lclk <= #1 clk66;
      _20_MHz:   to_be_lclk <= #1 int_clks[0];
      _10_MHz:   to_be_lclk <= #1 int_clks[1];
      _5_MHz:    to_be_lclk <= #1 int_clks[2];
      _2_5_MHz:  to_be_lclk <= #1 int_clks[3];
      _1_25_MHz: to_be_lclk <= #1 int_clks[4];
      _40_Hz:    to_be_lclk <= #1 int_clks[19];  // 38.15 Hz
      _2_Hz:     to_be_lclk <= #1 int_clks[23];  // 2.38 Hz
      _1_Hz:     to_be_lclk <= #1 int_clks[24];  // 1.19 Hz
      _half_Hz:  to_be_lclk <= #1 int_clks[25];  // 0.6 Hz
      default:   to_be_lclk <= #1 1'b x;
    endcase 
  
endmodule

/*
`ifdef exemplar

module sync_rotary(clk, out, in);
  input clk;
  input [3:0] in;
  output [3:0] out;
endmodule

`endif
*/

