/* @(#)muxer_merger.v	4.2

   Project: Altera circuit of SLIDAS 

   File name: muxer_merger.v
   Version:   4.2
   Updated:   99/08/26 13:56:29

   Author: Ph. Brodier-Yourstone, ECP division, CERN


   
-------------------------------------------------------------------------------
     SCCS    |          |             |
    Version  |  Author  |  Mod. Date  |          Changes made
-------------------------------------------------------------------------------
     1.1     |   PBY    | 14-02-1997  | - Started SCCS history (but the file
             |          |             | was created some days before)
             |          |             | 
     1.2     |   PBY    | 26-02-1997  | - no changes at all.
             |          |             |
     1.3     |   PBY    | 05-03-1997  | - Added comments
             |          |             |
     1.4     |   PBY    | 06-04-1997  | - Added even more comments
             |          |             | 
-------------------------------------------------------------------------------



     Module description:
     -------------------

     This module acts mainly as a multiplexer in data_gen. It has 2 32-bit
     input and 1 32-bit output. The selection input consists of 2 lines and
     not 1, as one might think.

     When one of the selection bits is set to 1, one of the inputs is
     forwarded to the output. When none is active the output is set to 0
     (this will be the value seen on the output lines during a reset, as
     the state_machine will have all 0 on its 'selection' bus).

     When both selection bits are set to 1 that means that the state machine
     is selection a control word. This is where this module merges the two
     inputs: it takes the data bits from one and the error information from
     the other.

/////////////////////////////////////////////////////////////////////////////
*/


module muxer_merger (lclk, sysenable, sel, in0, in1, q);
  input lclk, sysenable;
  input [1:0] sel;
  input [31:0] in0, in1;
  output [31:0] q;
  reg [31:0] q;

  always @(posedge lclk)
    if (sysenable)
      case (sel)
	2'b 10:  q <= #1 in0;
		      
	2'b 01:  q <= #1 in1;
		      
	2'b 11:  q <= #1 {in0[31:4], in1[3:0]};
		       
	default: q <= #1 32'b 0;
      endcase
endmodule

  
