/*
/////////////////////////////////////////////////////////////////////////////
//
//   Project: Altera circuit of SLIDAS 
//
//   File name: xoff_handler.v
//   Version:   4.2
//   Updated:   99/08/26 13:56:47
//
//   Author: Ph. Brodier-Yourstone, ECP division, CERN
//
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////

     Module description:
     -------------------
     
     This module translates uxoff_n into sysenable. The switch 'femb'
     decides whether to consider uxoff_n as a synchronous or an asynchronous
     signal.
     If femb == 1, uxoff_n is considered to be synchronous (used when slidas
     acts as a femb) and uxoff_n is synchronised one stage before becoming
     sysenable.
     If femb == 0, uxoff_n is considered to be asynchronous and has to
     go through a synchronisation stage.

/////////////////////////////////////////////////////////////////////////////
*/


module xoff_handler (lclk, sysenable, sysenable2, uxoff_n, femb);
  input lclk, uxoff_n, femb;
  output sysenable, sysenable2;

  reg uxoff_s1, uxoff_s2;
  wire sysenable, sysenable2;


  always @(posedge lclk)
    begin
      uxoff_s1 <= #1 uxoff_n;
      uxoff_s2 <= #1 uxoff_s1;
    end

ff_w_enable ff1 (lclk, sysenable, femb?uxoff_n:uxoff_s2, 1'b1);

ff_w_enable ff2 (lclk, sysenable2, femb?uxoff_n:uxoff_s2, 1'b1);

//  always @(posedge lclk) sysenable  <= #1 femb?uxoff_n:uxoff_s2;
    
//  always @(posedge lclk) sysenable2 <= #1 femb?uxoff_n:uxoff_s2;

endmodule

/*  
`ifdef exemplar
    // if during compilation with Galileo

module ff_w_enable (lclk, out, in, enable);
  input lclk, in, enable;
  output out;
endmodule

`else
// this line should be uncommented if you want to do simulation at a lower
// level than altera.v
// `include "ff_w_enable.v"
`endif
*/

