/*
/////////////////////////////////////////////////////////////////////////////
//
//   Project: Altera circuit of SLIDAS 
//
//   File name: power_on_reset.v
//   Version:   4.2
//   Updated:   99/08/26 13:56:32
//
//   Author: Ph. Brodier-Yourstone, ECP division, CERN
//
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////

     Module description:
     -------------------

     Creates a signal (power_on_reset_n) that is high for 16 clock cycles
     right after power up. If udw is illegal (11), power_on_reset_n will
     be blocked at a low level and keep the SLIDAS in a reset state
     indefinitely.

/////////////////////////////////////////////////////////////////////////////
*/


module power_on_reset (lclk, power_on_reset_n, udw);
  input lclk;
  output power_on_reset_n;
  input [1:0] udw;
  
  reg power_on_reset_n;
  reg [3:0] power_on_counter;


/*======================================================================
After power on, the signal power_on_reset will be active for 16 cycles
of lclk. This signal is used to generate the signal ldown_n at the start
up of the Slidas.
======================================================================*/

  always @(posedge lclk) begin
    if (udw == 2'b11) power_on_counter <= #1 0; // udw in illegal position
      else if (power_on_counter != 4'b 1111)
      	power_on_counter <= #1 power_on_counter + 1;
      
    power_on_reset_n <= #1 (power_on_counter == 4'b 1111)?1:0;
  end
  
endmodule
