/* @(#)reset_blinker.v	4.2

   Project: Altera circuit of SLIDAS 

   File name: reset_blinker.v
   Version:   4.2
   Updated:   99/08/26 13:56:33

   Author: Ph. Brodier-Yourstone, ECP division, CERN


   
-------------------------------------------------------------------------------
     SCCS    |          |             |
    Version  |  Author  |  Mod. Date  |          Changes made
-------------------------------------------------------------------------------
     1.1     |   PBY    | 14-02-1997  | - Started SCCS history (but the file
             |          |             | was created some days before)     
             |          |             |
     1.2     |   PBY    | 28-02-1997  | - Modified something (I don't remember
             |          |             | why)
             |          |             |
     1.3     |   PBY    | 05-03-1997  | - Added comments
             |          |             | 
     1.4     |   PBY    | 06-04-1997  | - Added comments
             |          |             |
-------------------------------------------------------------------------------



     Module description:
     -------------------

     The sole purpose of this module is to provide a delay long enough
     so that a small URESET# pulse still can be visible on the LED.

/////////////////////////////////////////////////////////////////////////////
*/


module reset_blinker(clk66, ureset_n, reset_led);
  input clk66, ureset_n;
  output reset_led;

  reg reset_led;

  reg reset_s1, reset_s2, reset_p;
  
  reg carry0, carry1;

  reg [23:0] counter;

initial begin    // just for simulation purposes
  reset_led = 0;
  reset_s1  = 0;
  reset_s2  = 0;
  reset_p   = 0;
  carry0    = 0;
  carry1    = 0;
  counter   = 0;
end


// synchronise the ureset_n input and issue a pulse on a high-to-low level
// transition on ureset_n
  always @(posedge clk66)
    begin
      reset_s1 <= #1 ~ureset_n;
      reset_s2 <= #1 reset_s1;
      reset_p <= #1 reset_s1 & ~reset_s2;
    end

    
// necessary for getting a faster timing in the Altera
  always @(posedge clk66) begin
    carry0 <= #1 (counter[7:0] == 8'h 01)?1:0;
    carry1 <= #1 (counter[15:8] == 8'h 00)?1:0;
  end
    


//-----------------------------------------------------------------------------
// at reset, the counter is set to 0.
// upon a reset pulse, the counter is loaded with a high value (such as
// counter[23] is set to 1). The counter then counts down until counter[23]
// is cleared. counter[23] is used to create the LED pulse.
//  
  always @(posedge clk66)
    if (reset_p) counter <= #1 24'h ffffff;
      else if (counter[23]) begin
	
	counter[7:0] <= #1 counter[7:0] - 1;        // This implements a fast
	if (carry0)                                 // 24-bit counter.
	  counter[15:8] <= #1 counter[15:8] - 1;    // I believe now that with
	if (carry1 & carry0)                        // the Altera this is not
	  counter[23:16] <= #1 counter[23:16] - 1;  // really necessary
	  
      end

  always @(posedge clk66) reset_led <= #1 ~ureset_n | counter[23];
//-----------------------------------------------------------------------------

endmodule




