/*
///////////////////////////////////////////////////////////////////////////////
//
//   Project: Altera circuit of SLIDAS 
//
//   File name: data_counter.v
//   Version:   4.2
//   Updated:   99/08/26 13:56:18
//
//   Author: Ph. Brodier-Yourstone, ECP division, CERN
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////

     Module description:
     -------------------
     
     16-bit synchronous counter with synchronous enable and synchronous reset.
     The reset is actually implemented as a 'load' of an external value,
     which is set to 0. This trick gives better results in terms of internal
     delays when using galileo.
     
///////////////////////////////////////////////////////////////////////////////
*/


module data_counter (lclk, sysenable, reset, count_up, d, q);
  input lclk, sysenable, reset, count_up;
  input [15:0] d;
  output [31:0] q;
  reg [15:0] counter;
  
  assign q = {16'b0, counter};
  
  always @(posedge lclk)
    if (sysenable)
      if (reset) counter <= #1 d;
      else if (count_up)
	counter <= #1 counter + 1;
endmodule
