/*
/////////////////////////////////////////////////////////////////////////////
//
//   Project: Altera circuit of SLIDAS 
//   Data format SCHWICK
//   File name: look_up_table4.v
//   Version:   1.1
//   Updated:   8/7/99  
//
//   Author: Ph. Brodier-Yourstone, ECP division, CERN
//
////////////////////////////////////////////////////////////////////////////

------------------------------------------------------------------------------
SCCS     |	    |		  |
version  |  Author  |  Mod. Date  |         Changes made
------------------------------------------------------------------------------
  1.1    |   EB     |  31/8/99    |  Based on look_up_table.v version 4.2
	 |	    |		  |
------------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////

     Module description:
     -------------------
     This module implements a clocked memory of 16 32-bit words.

     If more data words should be needed, make the following
     changes:

     1)
     input [3:0] index;
         becomes
     input [4:0] index;

     2) uncomment defined by words 16 to 31.

/////////////////////////////////////////////////////////////////////////////
*/


     
module look_up_table (lclk, sysenable, index, q, pattern0_jumper_on);
  input lclk, sysenable, pattern0_jumper_on;
  input [3:0] index;
  output [31:0] q;

  wire [31:0] q;
  reg  [31:0] next_q;


// must use flip-flops as a module, to force the compiler to use
// sysenable as pure clock enable.
  
  ff_w_enable_32 ff_look_up (lclk, q, next_q, sysenable);


  always @(index or pattern0_jumper_on)
    case (index)
      0:  next_q <= #1 32'h00000000;  // Dedicated for all 0.
    /* used for Status words 1 & 3, for the status block position, and sometimes for the no. of status words. */
      
      1:  next_q <= #1 32'h00000001;  // status block position
      2:  next_q <= #1 32'h00000002;  // number of status words
      3:  next_q <= #1 32'h00000000;  // unused
      4:  next_q <= #1 32'h00000004;  // used for status word 2
      5:  next_q <= #1 32'h00000020;  // used for header size
      6:  next_q <= #1 32'heeeeeeee;  // start of header
      7:  next_q <= #1 32'h74010101;  // source identifier MUCTPI
      8:  next_q <= #1 32'hb0f00000;  // Used for start of block
      9:  next_q <= #1 32'h08000040;  // Format Version Number (4.0)
      10: next_q <= #1 32'h000000de;  // detector event type
      11: next_q <= #1 32'h00000000;  // unused
      12: next_q <= #1 32'he0f00000;  // Used for end of block
      13: next_q <= #1 32'h00000000;  // unused
      14: next_q <= #1 32'h00000000;  // unused
      15: next_q <= #1 32'h00000000;  // unused

/*    16: next_q <= #1 32'h0;      // 16 - 31 are all unused at present
      17: next_q <= #1 32'h08878787;
      18: next_q <= #1 32'h0;
      19: next_q <= #1 32'h0;
      20: next_q <= #1 32'hcccccccc;
      21: next_q <= #1 32'h0;
      22: next_q <= #1 32'h0;
      23: next_q <= #1 32'h12345678;
      24: next_q <= #1 32'haaaaaaaa;
      25: next_q <= #1 32'h23456789;
      26: next_q <= #1 32'h34567890;
      27: next_q <= #1 32'h0;
      28: next_q <= #1 32'h0;
      29: next_q <= #1 32'h0;
      30: next_q <= #1 32'h0;
      31: next_q <= #1 32'h0; 
*/
    endcase
endmodule



/*
`ifdef exemplar
    // if during compilation with Galileo

module ff_w_enable_32 (lclk, out, in, enable);
  input lclk, enable;
  input [31:0] in;
  output [31:0] out;
endmodule

`else
// this line should be uncommented if you want to do simulation at a lower
// level than altera.v
// `include "ff_w_enable.v"
`endif
*/

