/*
/////////////////////////////////////////////////////////////////////////////
//
//   Project: Altera circuit of SLIDAS 
//
//   File name: switches.v
//   Version:   4.2
//   Updated:   99/08/26 13:56:39
//
//   Author: Ph. Brodier-Yourstone, ECP division, CERN
//
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////

     Module description:
     -------------------


/////////////////////////////////////////////////////////////////////////////
*/


/* This module is the interface to the switches on the module.
Here the signals are synchronised and a level or a pulse are generated
as appropriate (a pulse for the push-buttons, a level for the switches)
======================================================================
*/

module switches (lclk,
                 inj_err,  async_inj_err2,  async_inj_err3,
		 step,     async_step2,     async_step3,
		 ldown,    async_ldown2,    async_ldown3,
		 data_err, async_data_err2, async_data_err3,
		 ctrl_err, async_ctrl_err2, async_ctrl_err3,
		 femb,     async_femb2,     async_femb3,
		 block,    async_block2,    async_block3,

		 async_pattern_switch, async_duty_switch,
		 async_speed_switch, async_length_switch,

 		 pattern_switch, duty_switch, speed_switch, length_switch,

		 sysreset);



input lclk;
input async_inj_err2, async_inj_err3, async_step2, async_step3;
input async_ldown2, async_ldown3, async_data_err2, async_data_err3;
input async_ctrl_err2, async_ctrl_err3, async_femb2, async_femb3;
input async_block2, async_block3;
input [3:0] async_pattern_switch, async_duty_switch;
input [3:0] async_speed_switch, async_length_switch;

output inj_err, step, ldown, data_err, ctrl_err, femb, block;
output [3:0] pattern_switch, duty_switch, speed_switch, length_switch;
output sysreset;

wire [3:0] pattern_switch, duty_switch, speed_switch, length_switch;
reg sysreset;

//======================================================================
//         Push buttons
//======================================================================

debouncer inj_err_deb    (inj_err_d,  async_inj_err2, async_inj_err3);
edge_latch PB_latch1     (inj_err_l, inj_err_d, inj_err_s);
sync_level inj_err_sync  (lclk, inj_err_s, inj_err_l);
sync_pulse inj_err_pulse (lclk, inj_err,   inj_err_s);

debouncer step_deb       (step_d,     async_step2,     async_step3);
edge_latch PB_latch2     (step_l, step_d, step_s);
sync_level step_sync     (lclk, step_s, step_l);
sync_pulse step_pulse    (lclk, step,   step_s);



//======================================================================
//         Switches
//======================================================================

debouncer ldown_deb    (ldown_d,    async_ldown3,    async_ldown2);
sync_level ldown_sync  (lclk, ldown, ldown_d);

debouncer data_err_deb (data_err_d, async_data_err3, async_data_err2);
sync_level data_err_sync(lclk, data_err, data_err_d);

debouncer ctrl_err_deb (ctrl_err_d, async_ctrl_err3, async_ctrl_err2);
sync_level ctrl_err_sync(lclk, ctrl_err, ctrl_err_d);

debouncer femb_deb     (femb,     async_femb2,     async_femb3);


debouncer block_deb    (block_d,    async_block3,    async_block2);
sync_level block_sync  (lclk, block, block_d);



//======================================================================
//         Rotary switches
//======================================================================

sync_rotary   pattern_switch_sync(lclk, pattern_switch, async_pattern_switch);
detect_change pattern_switch_ch(lclk, pattern_switch_change, pattern_switch);

sync_rotary   duty_switch_sync(lclk, duty_switch, async_duty_switch);
detect_change duty_switch_ch(lclk, duty_switch_change, duty_switch);

sync_rotary   speed_switch_sync(lclk, speed_switch, async_speed_switch);
detect_change speed_switch_ch(lclk, speed_switch_change, speed_switch);

sync_rotary   length_switch_sync(lclk, length_switch, async_length_switch);
detect_change length_switch_ch(lclk, length_switch_change, length_switch);



always @(posedge lclk)
  sysreset <= #1 (pattern_switch_change |
                  duty_switch_change |
		  speed_switch_change|
		  length_switch_change)?  1:0;

endmodule

/*
`ifdef exemplar

module sync_rotary(clk, out, in);
  input clk;
  input [3:0] in;
  output [3:0] out;
endmodule

module sync_level(lclk, out, in);
  input lclk, in;
  output out;
endmodule

module sync_pulse(lclk, out, in);
  input lclk, in;
  output out;
endmodule

module detect_change(lclk, detected_change, in);
  input lclk;
  input [3:0] in;
  output detected_change;
endmodule

module debouncer(out, i1, i2);
  input i1, i2;
  output out;
endmodule

module edge_latch (q, edge_in, reset);
  input edge_in, reset;
  output q;
endmodule

`else
*/

`include "sync_rotary.v"
`include "sync_level.v"
`include "sync_pulse.v"
`include "detect_change.v"
`include "debouncer.v"
`include "edge_latch.v"





