/*
/////////////////////////////////////////////////////////////////////////////
//
//   Project: Altera circuit of SLIDAS 
//
//   File name: walking_and_inverting.v
//   Version:   4.2
//   Updated:   99/08/26 13:56:45
//
//   Author: Ph. Brodier-Yourstone, ECP division, CERN
//
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////

     Module description:
     -------------------

     Generates Walking 1, walking 0, FF/00 or AA/55 depending on
     pattern_switch.

/////////////////////////////////////////////////////////////////////////////
*/


`define  test_mode        0
`define  walking_1        1
`define  walking_0        2
`define  FFFF_0000        3
`define  AAAA_5555        4
`define  random_data      5
`define  random_no_ctrl   6
`define  random_all_ctrl  7
`define  ROB_sel_lenghts  8
`define  ROB_4_lengths    9


`define _8_bits  2
`define _16_bits 1
`define _32_bits 0


module walking_and_inverting (lclk, sysenable, reset, enable,
                              pattern_switch, udw, q);

  input lclk, sysenable, reset, enable;
  input [1:0] udw;
  input [3:0] pattern_switch;
  output [31:0] q;

  reg [31:0] reset_value, q;

  reg [1:0] type;

  reg shifted_out;


//-----------------------------------------------------------------------------
// making pattern_switch to become a 2-bit value, to optimise timing and
// logic later on.
//   
  always @(posedge lclk)
    case (pattern_switch)
      `walking_0: type <= #1 0;
      
      `test_mode,
      `walking_1: type <= #1 1;
      
      `FFFF_0000: type <= #1 2;
      
      `AAAA_5555: type <= #1 3;
    endcase
//-----------------------------------------------------------------------------
    
    
//-----------------------------------------------------------------------------
// The reset value is different for each of the modes. By just inverting
// or shifting, the appropriate pattern will then be obtained.
//
  always @(posedge lclk)
    case (type)
      0: reset_value <= #1 ~32'b1;
      1: reset_value <= #1 32'b1;
      2: reset_value <= #1 32'hffffffff;
      3: reset_value <= #1 32'haaaaaaaa;
    endcase
//-----------------------------------------------------------------------------


    
//-----------------------------------------------------------------------------
// shifted_out is used during walking 1 or walking 0. It is used to shift
// the outgoing walking 1 or 0 back to bit 0. Depending on the data width
// defined by UDW, the bit is shifted back sooner or later.
//
// Handling this bit this way instead of shifting it directly from the
// shifting register simplifies the logic in next block.
// 
  always @(posedge lclk)
    if (sysenable)
      if (reset) shifted_out <= #1 reset_value[31];
      else if (enable)
	case (udw)
	  `_8_bits:  shifted_out <= #1 q[6];
	  `_16_bits: shifted_out <= #1 q[14];
	  default:   shifted_out <= #1 q[30];
	endcase
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Here the register is simply shifted or inverted, depending on the mode
// defined by pattern_switch (type). reset_value has previously been
// assigned the value corresponding to the current mode.
//	
  always @(posedge lclk)
    if (sysenable)
      if (reset) q <= #1 reset_value;
      else if (enable)
	case (type)
	  0, 1: q <= #1 {q[30:0], shifted_out};
	  2, 3: q <= #1 ~q;
	endcase
//-----------------------------------------------------------------------------
	
	
endmodule







